!{\src2tex{textfont=tt}}
!!****m* ABINIT/m_build_info
!! NAME
!!  m_build_info
!!
!! FUNCTION
!!  This module contains information about this particular version of ABINIT
!!  and its build parameters (useful for debugging).
!!
!! COPYRIGHT
!!  Copyright (C) 2005-2010 ABINIT group (Yann Pouillon, Matteo Giantomassi)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

module m_build_info

 use defs_basis

 implicit none

! Try to prevent problems with the length of the argument 
! (should not exceed the line length of 132 character)
! So, always start the string to be replaced, at the beginning of a line

! Parameters set-up by Autoconf
 character(len=6),parameter :: abinit_version = &
& "6.0.4"
 character(len=*),parameter :: build_target   = &
& "i586_linux_gnu4.3"

! More info on current version
 character(len=*),parameter :: version_major = &
& "6"
 character(len=*),parameter :: version_minor = &
& "0"
 character(len=*),parameter :: version_micro = &
& "4"
 character(len=*),parameter :: version_build = &
& "20130417"

 ! Info on compilers. Try to prevent problems with the length of the argument 
 ! (should not exceed the line length of 132 character)
 character(len=*),parameter :: cc_info    = &
& "gnu"
 character(len=*),parameter :: cc_flags   = &
& "-pipe -Wall -g -O2 -march=i586 -mtune=generic"
 character(len=*),parameter :: cxx_info   = &
& "generic0.0"
 character(len=*),parameter :: cxx_flags  = &
& "-pipe -Wall -g -O2 -march=i586 -mtune=generic"
 character(len=*),parameter :: fc_info    = &
& "gnu4.3"
 character(len=*),parameter :: fc_flags   = &
& "  -g  -ffree-line-length-none "
 character(len=*),parameter :: fc_ldflags = &
& "     "

 ! Info on optimizations
 character(len=*),parameter :: enable_debug    = &
& "yes"
 character(len=*),parameter :: enable_optim = &
& "standard"
 character(len=*),parameter :: cpu_info = &
& "_"

 ! Info on MPI
 character(len=*),parameter :: enable_mpi    = &
& "no"
 character(len=*),parameter :: enable_mpi_io = &
& "no"

 ! Info on BLAS and LAPACK
 character(len=*),parameter :: linalg_type      = &
& "abinit"
 character(len=*),parameter :: enable_scalapack = &
& "no"

 ! Info on plug-ins
 character(len=*),parameter :: enable_bigdft    = &
& "no"
 character(len=*),parameter :: enable_etsf_io   = &
& "no"
 character(len=*),parameter :: enable_libxc   = &
& "no"
 character(len=*),parameter :: enable_fox       = &
& "no"
 character(len=*),parameter :: enable_netcdf    = &
& "no"
 character(len=*),parameter :: enable_wannier90 = &
& "no"

 ! Info on experimental features
 character(len=*),parameter :: enable_bindings = &
& "no"
 character(len=*),parameter :: enable_error_handlers = &
& "no"
 character(len=*),parameter :: enable_exports = &
& "no"
 character(len=*),parameter :: enable_gw_dpc = &
& "no"
 character(len=*),parameter :: enable_macroave = &
& "yes"
 
#if defined HAVE_BZR_BRANCH
 ! Info on Bazaar branch (if applies)
 character(len=*),parameter :: bzr_branch = &
& ""
 character(len=*),parameter :: bzr_revno  = &
& ""
 character(len=*),parameter :: bzr_clean  = &
& ""
#endif

contains  !===========================================================
!!***

!!****f* ABINIT/dump_config
!! NAME
!!  dump_config
!!
!! FUNCTION
!!  Reports a printout of the information stored in m_build_infos,
!!  useful for error messages and debugging.
!!
!! INPUTS
!!  unit=the fortran unit number (default is std_out)
!!
!! OUTPUT
!!  Only printing
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

subroutine dump_config(my_unit)

 use defs_basis

 implicit none

!Arguments ------------------------------------
 integer,optional,intent(in) :: my_unit

!Local variables-------------------------------
 integer :: unt
! *********************************************************************
 
 !TODO things that might be added through preprocessing options: date and time of compilation 

 unt=std_out
 if (PRESENT(my_unit)) unt=my_unit

 write(unt,*)
 write(unt,'(1x,a)') REPEAT('+',78)
 write(unt,*)
 write(unt,'(a)' )' === Build Information === '
 write(unt,'(2a)')'  Version       : ',TRIM(abinit_version)
 write(unt,'(2a)')'  Build target  : ',TRIM(build_target)
 write(unt,'(2a)')'  Build date    : ',TRIM(version_build)
 write(unt,*)
 write(unt,'(a)' )' === Compiler Suite === '
 write(unt,'(2a)')'  C compiler       : ',TRIM(cc_info)
 write(unt,'(2a)')'  CFLAGS           : ',TRIM(cc_flags)
 write(unt,'(2a)')'  C++ compiler     : ',TRIM(cxx_info)
 write(unt,'(2a)')'  CXXFLAGS         : ',TRIM(cxx_flags)
 write(unt,'(2a)')'  Fortran compiler : ',TRIM(fc_info)
 write(unt,'(2a)')'  FCFLAGS          : ',TRIM(fc_flags)
 write(unt,'(2a)')'  FC_LDFLAGS       : ',TRIM(fc_ldflags)
 write(unt,*)
 write(unt,'(a) ')' === Optimizations === '
 write(unt,'(2a)')'  Debug level        : ',TRIM(enable_debug)
 write(unt,'(2a)')'  Optimization level : ',TRIM(enable_optim)
 write(unt,'(2a)')'  Architecture       : ',TRIM(cpu_info)
 write(unt,*)
 write(unt,'(a) ')' === MPI === '
 write(unt,'(2a)')'  Parallel build : ',TRIM(enable_mpi)
 write(unt,'(2a)')'  Parallel I/O   : ',TRIM(enable_mpi_io)
 write(unt,*)
 write(unt,'(a) ')' === Linear algebra === '
 write(unt,'(2a)')'  Library type  : ',TRIM(linalg_type)
 write(unt,'(2a)')'  Use ScaLAPACK : ',TRIM(enable_scalapack)
 write(unt,*)
 write(unt,'(a)' )' === Plug-ins === '
 write(unt,'(2a)')'  BigDFT    : ',TRIM(enable_bigdft)
 write(unt,'(2a)')'  ETSF I/O  : ',TRIM(enable_etsf_io)
 write(unt,'(2a)')'  LibXC     : ',TRIM(enable_libxc)
 write(unt,'(2a)')'  FoX       : ',TRIM(enable_fox)
 write(unt,'(2a)')'  NetCDF    : ',TRIM(enable_netcdf)
 write(unt,'(2a)')'  Wannier90 : ',TRIM(enable_wannier90)
 write(unt,*)
 write(unt,'(a)' )' === Experimental features === '
 write(unt,'(2a)')'  Bindings            : ',TRIM(enable_bindings)
 write(unt,'(2a)')'  Error handlers      : ',TRIM(enable_error_handlers)
 write(unt,'(2a)')'  Exports             : ',TRIM(enable_exports)
 write(unt,'(2a)')'  GW double-precision : ',TRIM(enable_gw_dpc)
 write(unt,'(2a)')'  Macroave build      : ',TRIM(enable_macroave)
 write(unt,*)
#if defined HAVE_BZR_BRANCH
 write(unt,'(a)' )' === Bazaar branch information === '
 write(unt,'(2a)')'  Branch ID : ',TRIM(bzr_branch)
 write(unt,'(2a)')'  Revision  : ',TRIM(bzr_revno)
 write(unt,'(2a)')'  Committed : ',TRIM(bzr_clean)
 write(unt,*)
#endif
 write(unt,'(1x,a)') REPEAT('+',78)
 write(unt,*)

end subroutine dump_config

end module m_build_info
!!***
