!{\src2tex{textfont=tt}}
!!****f* ABINIT/splint
!! NAME
!! splint
!!
!! FUNCTION
!!  Compute spline interpolation. There is no hypothesis
!!  about the spacing of the input grid points.
!!
!! COPYRIGHT
!! Copyright (C) 2009-2010 ABINIT group.
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  nspline: number of grid points of input mesh
!!  xspline(nspline): input mesh
!!  yspline(nspline): function on input mesh
!!  ysplin2(nspline): second derivative of yspline on input mesh
!!  nfit: number of points of output mesh
!!  xfit(nfit): output mesh
!!
!! OUTPUT
!!  yfit(nfit): function on output mesh
!!
!! TODO
!!
!! PARENTS
!!      atomden,calc_sig_noppm,cc_derivatives,denfgr,m_atom,m_paw_slater
!!      partial_dos_fractions,pawdij0,pawgylm,pawkij,pawmkaewf,psp6cc,psp7cc
!!      psp7in,psp9in,spline_paw_fncs,splint_complex,vso_realspace_local,wffile
!!
!! CHILDREN
!!
!! SOURCE
 subroutine splint (nspline,xspline,yspline,ysplin2,&
&                  nfit,xfit,yfit)


 implicit none

 integer, intent(in) :: nfit, nspline
 double precision, intent(in) :: xspline(nspline)
 double precision, intent(in) :: yspline(nspline)
 double precision, intent(in) :: ysplin2(nspline)
 double precision, intent(in) :: xfit(nfit)

 double precision, intent(out) :: yfit(nfit)


!local
 integer :: left,i,k,right
 double precision :: delarg,invdelarg,aa,bb

!source

 left = 1
 do i=1, nfit

! Initialize for the unlikely event that rmax exceed r(mesh)
  yfit(i)=0.d0

  do k=left+1, nspline
   if(xspline(k) >= xfit(i)) then
    if(xspline(k-1) <= xfit(i)) then
     right = k
     left = k-1
    else
     if (k-1.eq.1 .and. i.eq.1) then
      stop '  splint: xfit(1) < xspline(1)'
     else
      stop '  splint: xfit not properly ordered'
     end if
    end if
    delarg= xspline(right) - xspline(left)
    invdelarg= 1.0d0/delarg
    aa= (xspline(right)-xfit(i))*invdelarg
    bb= (xfit(i)-xspline(left))*invdelarg

    yfit(i) = aa*yspline(left) + bb*yspline(right)    &
&            +( (aa*aa*aa-aa)*ysplin2(left) +         &
&               (bb*bb*bb-bb)*ysplin2(right) ) *delarg*delarg/6.0d0
    exit
   end if
  end do ! k
 end do ! i

 return
 end


!!***

