!{\src2tex{textfont=tt}}
!!****f* ABINIT/ep_setupqpt
!!
!! NAME
!! ep_setupqpt
!!
!! FUNCTION
!!  set up qpoint grid for elphon.
!!  2 modes, either uniform grid from anaddb input npqpt
!!  or take qpt from anaddb input (explicitly listed)
!!
!! COPYRIGHT
!! Copyright (C) 2009-2010 ABINIT group (MVer, )
!! This file is distributed under the terms of the
!! GNU General Public Licence, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   anaddb_dtset=dataset with input variables
!!     %qgrid_type gives type of q grid 1=uniform 2=take from input
!!     %ep_nqpt    number of auxiliary qpoints 
!!     %ep_qptlist list of qpoints, 
!!
!! OUTPUT
!!   
!! NOTES
!!   
!! PARENTS
!!      elphon
!!
!! CHILDREN
!!      getkgrid,leave_new,smpbz,wrap2_pmhalf,wrtout
!!
!! NOTES
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine ep_setupqpt (anaddb_dtset,elph_ds,nsym,qptrlatt,rprimd,symrel)

 use defs_basis
 use defs_datatypes
 use defs_abitypes
 use defs_elphon

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_14_hidewrite
 use interfaces_16_hideleave
 use interfaces_32_util
 use interfaces_56_recipspace
!End of the abilint section

 implicit none

!Arguments -------------------------------

 integer, intent(in) :: nsym
 integer, intent(in) :: symrel(3,3,nsym)
 type(anaddb_dataset_type), intent(in) :: anaddb_dtset

 real(dp), intent(in) :: rprimd(3,3)

 type(elph_type), intent(inout) :: elph_ds
 integer, intent(out) :: qptrlatt(3,3) 

!Local variables -------------------------

 integer :: nqshft,option,iqpt
 integer :: iscf,mqpt,istat,iout,berryopt,nqpt_computed
 real(dp) :: qptrlen, kpt(3), res

 integer :: symafm(nsym),vacuum(3)

 real(dp), allocatable :: wtq(:),qpt_full(:,:),tmpshifts(:,:)
 character(len=500) :: message

! *********************************************************************

!default is to expect a uniform grid
 elph_ds%tuniformgrid = 1


!if we use the normal grid way of generating the qpoints:
 if (anaddb_dtset%qgrid_type==1) then
!  qpoint lattice vectors (inverse, like kptrlatt)
   qptrlatt(:,:)=0
   qptrlatt(1,1)=anaddb_dtset%ngqpt(1)
   qptrlatt(2,2)=anaddb_dtset%ngqpt(2)
   qptrlatt(3,3)=anaddb_dtset%ngqpt(3)
   
   if (anaddb_dtset%nqshft /= 1) then
!    MG Check this part, dont know if it works, never used shifted grids!
!    try to reduce the qpoint grid to a single qshift, otherwise stop
!    dummy args for call to getkgrid
     symafm(:) = 1
     vacuum(:) = 0
     iscf = 3
     
     mqpt = anaddb_dtset%ngqpt(1)*anaddb_dtset%ngqpt(2)*anaddb_dtset%ngqpt(3)*anaddb_dtset%nqshft
     allocate(qpt_full(3,mqpt),wtq(mqpt),tmpshifts(3,8),stat=istat)
     if (istat /= 0) stop 'elphon: error in allocating qpt_full(3,mqpt),wtq(mqpt),tmpshifts(3,8)'

     wtq(:) = one

     tmpshifts(:,:) = zero
     tmpshifts(:,1:4) = anaddb_dtset%q1shft(:,:)

     iout=6

     berryopt = 1

!    just call with identity, to get full set of kpts in qpt_full, but
!    reduce qshfts
     
     nqshft=anaddb_dtset%nqshft
     call getkgrid(iout,iscf,qpt_full,3,qptrlatt,qptrlen, &
&     1,mqpt,nqpt_computed,nqshft,1,rprimd,tmpshifts,symafm, &
&     symrel,vacuum,wtq)
     deallocate (qpt_full,wtq,tmpshifts)
     if (anaddb_dtset%nqshft /= 1) then
       write (message,'(6a,i4)')ch10,&
&       ' elphon : ERROR- ',ch10,&
&       ' multiple qpt shifts not treated yet',ch10,&
&       ' -- should be possible ', anaddb_dtset%nqshft
       call wrtout(std_out,message,'COLL')
       call leave_new('COLL')
     end if
   end if  ! end multiple shifted qgrid


   write(message,'(3a,9i3)')&
&   ' elphon : enter smpbz ',ch10,&
&   ' qptrlatt = ',qptrlatt 
   call wrtout(std_out,message,'COLL')

   option=1
!  mqpt=anaddb_dtset%ngqpt(1)*anaddb_dtset%ngqpt(2)*anaddb_dtset%ngqpt(3)*anaddb_dtset%nqshft
   mqpt= qptrlatt(1,1)*qptrlatt(2,2)*qptrlatt(3,3) &
&   +qptrlatt(1,2)*qptrlatt(2,3)*qptrlatt(3,1) &
&   +qptrlatt(1,3)*qptrlatt(2,1)*qptrlatt(3,2) &
&   -qptrlatt(1,2)*qptrlatt(2,1)*qptrlatt(3,3) &
&   -qptrlatt(1,3)*qptrlatt(2,2)*qptrlatt(3,1) &
&   -qptrlatt(1,1)*qptrlatt(2,3)*qptrlatt(3,2)

   allocate(qpt_full(3,mqpt),stat=istat)
   if (istat /= 0) stop 'elphon: error in allocating qpt_full'
   iout = 6
   call smpbz(anaddb_dtset%brav,iout,qptrlatt,mqpt,elph_ds%nqpt_full,anaddb_dtset%nqshft,option,anaddb_dtset%q1shft,qpt_full)


!  save the q-grid for future reference
   allocate(elph_ds%qpt_full(3,elph_ds%nqpt_full),stat=istat)
   if (istat /= 0) stop 'elphon: error in allocating elph_ds%qpt_full'

!  reduce qpt_full to correct zone
   do iqpt=1,elph_ds%nqpt_full
     call wrap2_pmhalf(qpt_full(1,iqpt),kpt(1),res)
     call wrap2_pmhalf(qpt_full(2,iqpt),kpt(2),res)
     call wrap2_pmhalf(qpt_full(3,iqpt),kpt(3),res)
     qpt_full(:,iqpt) = kpt
     elph_ds%qpt_full(:,iqpt)=kpt
   end do
   deallocate (qpt_full)

 else if (anaddb_dtset%qgrid_type==2) then ! use explicit list of qpoints from anaddb input
   qptrlatt(:,:)=0
   qptrlatt(1,1)=1
   qptrlatt(2,2)=1
   qptrlatt(3,3)=1

   elph_ds%nqpt_full=anaddb_dtset%ep_nqpt
   allocate(elph_ds%qpt_full(3,elph_ds%nqpt_full),stat=istat)
   if (istat /= 0) stop 'elphon: error in allocating elph_ds%qpt_full'
   
   elph_ds%qpt_full = anaddb_dtset%ep_qptlist

   elph_ds%tuniformgrid = 0

 end if ! type of qgrid for elphon

end subroutine ep_setupqpt
!!***
