!{\src2tex{textfont=tt}}
!!****f* ABINIT/get_gkk_qpt_tr
!!
!! NAME
!! get_gkk_qpt_tr
!!
!! FUNCTION
!!  calculate the (in) and (out) velocity factors for transport
!!
!! COPYRIGHT
!! Copyright (C) 2004-2010 ABINIT group (JPC)
!! This file is distributed under the terms of the
!! GNU General Public Licence, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  elph_ds
!!    elph_ds%nFSband = number of bands included in the FS integration
!!    elph_ds%nFSkpt = number of kpts included in the FS integration
!!    elph_ds%nFSband = number of bands included in the FS integration
!!    elph_ds%minFSband = index of the lowest FS band
!!    elph_ds%nqpt_full  = number of Q pts 
!!    elph_ds%nqptirred  = number of irreducible Q pts 
!!  to index the GS electronic states :
!!  hdr%nkpt = full number of k points 
!!  nband =full number of bands 
!!  FSfulltoirred = mapping of full FS kpts to irreducible ones
!!   FSfullpqtofull = mapping of k + q to k
!!   FSirredtoGS = mapping of irreducible kpoints to GS set
!! OUTPUT
!! elph_tr_ds%eta_trout = prefactor for out gkk_qpt
!! elph_tr_ds%eta_trin = prefactor for in_gkk_qpt
!! elph_tr_ds%FSelecveloc_sq = avergae FS electronic velocity
!! SIDE EFFECTS
!!
!! PARENTS
!!      elphon
!!
!! CHILDREN
!!      leave_new,read_el_veloc,wrtout
!!
!! NOTES
!!   
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine get_gkk_qpt_tr(elph_ds,mpi_enreg,nband,hdr,FSfulltoirred, FSirredtoGS,FSfullpqtofull,elph_tr_ds)

  use defs_datatypes
  use defs_abitypes
  use defs_elphon

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_14_hidewrite
 use interfaces_16_hideleave
 use interfaces_77_ddb, except_this_one => get_gkk_qpt_tr
!End of the abilint section

  implicit none


!Arguments ------------------------------------
  !scalars
  integer,intent(in) :: nband
  type(MPI_type), intent(inout) :: mpi_enreg

  !arrays
  type(hdr_type),intent(in) :: hdr
  type(elph_type),intent(in) :: elph_ds
  type(elph_tr_type):: elph_tr_ds


  integer,intent(in) :: FSfulltoirred(3,elph_ds%nFSkpt), FSirredtoGS(elph_ds%nFSkptirred)
  integer,intent(in) :: FSfullpqtofull(elph_ds%nFSkpt,elph_ds%nqpt_full)

!Local variables-------------------------------
  !scalars
  integer:: iqpt,iFSkpt,ikpt,iFSkptq,ikptpq
  integer:: ib1,ib2,fib1,fib2,isppol,ibeff,istat
  integer :: etasize
  real(dp)::eta2,etain,etaout
  !arrays
  real(dp):: elvelock(3),elvelockpq(3)
  character(len=fnlen) :: fname
  character(len=500) :: message
  integer::iost,ierr

  real(dp),allocatable :: tmp_etain(:,:,:)
  real(dp),allocatable :: tmp_etaout(:,:,:)

! *********************************************************************

!check inputs
!TODO: should be done at earlier stage of initialization and checking
 write (*,*) ' ngkkband,nFSband,minFSband = ', elph_ds%ngkkband,elph_ds%nFSband,elph_ds%minFSband
 if (elph_ds%ngkkband /= elph_ds%nFSband) then
   write (message,'(a)') 'Error: need to keep electron band dependency in memory for transport calculations'
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if

!crc 
 if (elph_ds%gkqwrite == 0) then
   write (*,*) 'Transport calculation: allocate eta_trin/out',&
&   ' and call get_gkk_qpt_tr'
   allocate (elph_tr_ds%eta_trin(elph_ds%ngkkband*elph_ds%ngkkband,&
&   elph_ds%nFSkpt,elph_ds%nsppol,elph_ds%nqptirred),stat=istat)
   if (istat /= 0) stop 'elphon: error in allocating eta_trin'
   allocate (elph_tr_ds%eta_trout(elph_ds%ngkkband*elph_ds%ngkkband,&
&   elph_ds%nFSkpt,elph_ds%nsppol,elph_ds%nqptirred),stat=istat)
   if (istat /= 0) stop 'elphon: error in allocating eta_trout'
!  crc
 end if


 elph_tr_ds%unitgkq_trin=40 
 elph_tr_ds%unitgkq_trout=41

 etasize = elph_tr_ds%onegkksize/elph_ds%nbranch/elph_ds%nbranch/2

 if (elph_ds%gkqwrite==1) then

   fname=trim(elph_ds%elph_base_name) // '_GKKQ_trin'
   write(6,*)'SIZES ', etasize
   open (unit=elph_tr_ds%unitgkq_trin,file=fname,access='direct',recl=etasize,&
&   form='unformatted',status='unknown',iostat=iost)

   if (iost /= 0) then
     write (message,'(2a)')' get_gkk_qpt_tr : ERROR- opening file ',trim(fname)
     call wrtout(std_out,message,'COLL')
     call leave_new('COLL')
   end if


   fname=trim(elph_ds%elph_base_name) // '_GKKQ_trout'
   open (unit=elph_tr_ds%unitgkq_trout,file=fname,access='direct',recl=etasize,&
&   form='unformatted',status='unknown',iostat=iost)

   if (iost /= 0) then
     write (message,'(2a)')' get_gkk_qpt_tr : ERROR- opening file ',trim(fname)
     call wrtout(std_out,message,'COLL')
     call leave_new('COLL')
   end if
 end if

 write(6,*)'reading of electronic velocities'
 allocate(elph_tr_ds%el_veloc(hdr%nkpt,nband,3,elph_ds%nsppol))
 call read_el_veloc(mpi_enreg,nband,hdr%nkpt,elph_ds%nsppol,elph_tr_ds)

!precalculate the Fermi speed modulus squared
 allocate (elph_tr_ds%FSelecveloc_sq(elph_ds%nsppol))
 elph_tr_ds%FSelecveloc_sq=0.
 do isppol=1,elph_ds%nsppol
   do iFSkpt=1,elph_ds%nFSkpt
     ikpt=FSirredtoGS(FSfulltoirred(1,iFSkpt))
     do ib1=1,elph_ds%nFSband
       fib1=ib1+elph_ds%minFSband-1
       elvelock(:)=elph_tr_ds%el_veloc(ikpt,fib1,:,isppol)
       eta2=elvelock(1)*elvelock(1)+elvelock(2)*elvelock(2)+elvelock(3)*elvelock(3)  

       elph_tr_ds%FSelecveloc_sq(isppol)=elph_tr_ds%FSelecveloc_sq(isppol)&
&       +eta2*elph_ds%FSintweight(ib1,iFSkpt,isppol)/elph_ds%nFSkpt
     end do
   end do
   elph_tr_ds%FSelecveloc_sq(isppol) =elph_tr_ds%FSelecveloc_sq(isppol)/elph_ds%n0(isppol)
   write (std_out,*) '  get_gkk_qpt_tr: FSelecveloc_sq ', elph_tr_ds%FSelecveloc_sq 
 end do ! end isppol


!allocate temp variables
 allocate (tmp_etain (elph_ds%ngkkband*elph_ds%ngkkband,elph_ds%nFSkpt,elph_ds%nsppol),stat=ierr)
 if (ierr /= 0 ) then
   write (message,'(3a)')' get_gkk_qpt_tr : ERROR- ',ch10,&
&   ' trying to allocate array tmp_etain '
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if

 allocate (tmp_etaout (elph_ds%ngkkband*elph_ds%ngkkband,elph_ds%nFSkpt,elph_ds%nsppol),stat=ierr)
 if (ierr /= 0 ) then
   write (message,'(3a)')' get_gkk_qpt_tr : ERROR- ',ch10,&
&   ' trying to allocate array tmp_etaout '
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if


!calculates the in and out factors

!MJV 31/03/2009: Note that the following is valid for any geometry, not just cubic!
!see eq 5 and 6 of prb 36 4103 (Al-Lehaibi et al 1987)
 
 do iqpt=1,elph_ds%nqptirred

   do isppol=1,elph_ds%nsppol
     do iFSkpt=1,elph_ds%nFSkpt
       ikpt=FSirredtoGS(FSfulltoirred(1,iFSkpt))        !k     
       iFSkptq = FSfullpqtofull(iFSkpt,iqpt)
       ikptpq=FSirredtoGS(FSfulltoirred(1,iFSkptq))     !k'=k+q

       do ib1=1,elph_ds%nFSband
         fib1=ib1+elph_ds%minFSband-1
         elvelock(:)=elph_tr_ds%el_veloc(ikpt,fib1,:,isppol)

         do ib2=1,elph_ds%nFSband
           ibeff=ib2+(ib1-1)*elph_ds%nFSband

           fib2=ib2+elph_ds%minFSband-1
           elvelockpq(:)= elph_tr_ds%el_veloc(ikptpq,fib2,:,isppol)

!          generalization to tensorial quantities should be simple, by keeping the directional
!          references of velock and velockpq as indices. The following is the trace of the tensor
           etain=elvelock(1)*elvelockpq(1)+elvelock(2)*elvelockpq(2)+elvelock(3)*elvelockpq(3)
           etaout=elvelock(1)*elvelock(1)+elvelock(2)*elvelock(2)+elvelock(3)*elvelock(3)
           etain =etain /elph_tr_ds%FSelecveloc_sq(isppol)
           etaout=etaout/elph_tr_ds%FSelecveloc_sq(isppol)

           tmp_etaout(ibeff,iFSkpt,isppol)= etaout
           tmp_etain(ibeff,iFSkpt,isppol)= etain

         end do
       end do
     end do
   end do

   if (elph_ds%gkqwrite == 0) then
     elph_tr_ds%eta_trin (:,:,:,iqpt) = tmp_etain
     elph_tr_ds%eta_trout(:,:,:,iqpt) = tmp_etaout
   else if (elph_ds%gkqwrite == 1) then
     write(6,*)'write 0'
     do iFSkpt=1, elph_ds%nFSkpt
       write (elph_tr_ds%unitgkq_trin,REC=((iqpt-1)*elph_ds%nFSkpt)+iFSkpt) tmp_etain(:,iFSkpt,:)
     end do
     write(6,*)'write in'
     do iFSkpt=1, elph_ds%nFSkpt
       write (elph_tr_ds%unitgkq_trout,REC=((iqpt-1)*elph_ds%nFSkpt)+iFSkpt) tmp_etaout(:,iFSkpt,:)
     end do
     write(6,*)'write out'
   end if

 end do !iq

 deallocate (tmp_etaout)
 deallocate (tmp_etain)

 deallocate(elph_tr_ds%el_veloc)
 write(6,*)'out of get_gkk_qpt_tr'
 return
end subroutine get_gkk_qpt_tr
!!***
