!{\src2tex{textfont=tt}}
!!****f* ABINIT/integrate_gamma_tr
!!
!! NAME
!! integrate_gamma_tr
!!
!! FUNCTION
!! This routine integrates the TRANSPORT electron phonon coupling matrices
!! over the kpoints on the fermi surface. A dependency on qpoint
!! remains for gamma_qpt_in/out
!! Copied from integrate_gamma
!!
!! COPYRIGHT
!! Copyright (C) 2004-2010 ABINIT group (JPC,MJV)
!! This file is distributed under the terms of the
!! GNU General Public Licence, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   elph_ds = elphon datastructure with data and dimensions
!!      elph_ds%qpt_full = qpoint coordinates
!!   FSfullpqtofull = mapping of k+q to k
!!   nrpt = number of real space points for FT
!!
!! OUTPUT
!!   elph_tr_ds%gamma_qpt_trout and created elph_tr_ds%gamma_rpt_trout
!!   elph_tr_ds%gamma_qpt_trin and created elph_tr_ds%gamma_rpt_trin
!!
!! NOTES
!!
!! PARENTS
!!      elphon
!!
!! CHILDREN
!!      leave_new,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine integrate_gamma_tr(elph_ds,FSfullpqtofull,nrpt,elph_tr_ds)

 use defs_basis
  use defs_datatypes
 use defs_abitypes
  use defs_elphon

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_14_hidewrite
 use interfaces_16_hideleave
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nrpt
 type(elph_tr_type) :: elph_tr_ds
 type(elph_type),intent(inout) :: elph_ds
!arrays
 integer,intent(in) :: FSfullpqtofull(elph_ds%nFSkpt,elph_ds%nqpt_full)

!Local variables-------------------------------
!scalars
 integer :: iFSkpt,iFSkptq,ib1,ib2,ibeff,ierr,iqpt,iqpt_fullbz,isppol
 character(len=500) :: message
!arrays
 real(dp),allocatable :: tmp_etain(:,:,:),tmp_etaout(:,:,:)
 real(dp),allocatable :: tmp_gkk(:,:,:,:,:)

! *************************************************************************

 allocate(elph_tr_ds%gamma_qpt_trin(2,elph_ds%nbranch*elph_ds%nbranch,elph_ds%nsppol,elph_ds%nqpt_full),stat=ierr)
 if (ierr /= 0 ) then
   write (message,'(3a)')' integrate_gamma_tr : ERROR- ',ch10,&
&   ' trying to allocate array elph_tr_ds%gamma_qpt_trin '
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if
 elph_tr_ds%gamma_qpt_trin(:,:,:,:) = zero

 allocate(elph_tr_ds%gamma_qpt_trout(2,elph_ds%nbranch*elph_ds%nbranch,elph_ds%nsppol,elph_ds%nqpt_full),stat=ierr)
 if (ierr /= 0 ) then
   write (message,'(3a)')' integrate_gamma_tr : ERROR- ',ch10,&
&   ' trying to allocate array elph_tr_ds%gamma_qpt_trout '
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if
 elph_tr_ds%gamma_qpt_trout(:,:,:,:) = zero

 allocate (elph_tr_ds%gamma_rpt_trout(2,elph_ds%nbranch*elph_ds%nbranch,elph_ds%nsppol,nrpt),stat=ierr)
 if (ierr /= 0 ) then
   write (message,'(3a)')' integrate_gamma_tr : ERROR- ',ch10,&
&   ' trying to allocate array elph_tr_ds%gamma_rpt_trout '
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if
 elph_tr_ds%gamma_rpt_trout(:,:,:,:) = zero

 allocate (elph_tr_ds%gamma_rpt_trin(2,elph_ds%nbranch*elph_ds%nbranch,elph_ds%nsppol,nrpt),stat=ierr)
 if (ierr /= 0 ) then
   write (message,'(3a)')' integrate_gamma_tr : ERROR- ',ch10,&
&   ' trying to allocate array elph_tr_ds%gamma_rpt_trin '
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if
 elph_tr_ds%gamma_rpt_trin(:,:,:,:) = zero


!information
 if (elph_ds%gkqwrite == 0) then
   write (message,'(a)')' integrate_gamma_tr : keeping gamma matrices in memory'
   call wrtout(std_out,message,'COLL')
 else if (elph_ds%gkqwrite == 1) then
   write (message,'(a)')' integrate_gamma_tr : reading gamma matrices from disk'
   call wrtout(std_out,message,'COLL')
 else
   write (message,'(3a,i3)')' integrate_gamma_tr : BUG-',ch10,&
&   ' Wrong value for gkqwrite = ',elph_ds%gkqwrite
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if

!allocate temp variables
 allocate (tmp_gkk (2,elph_ds%ngkkband*elph_ds%ngkkband,elph_ds%nbranch*elph_ds%nbranch,&
& elph_ds%nFSkpt,elph_ds%nsppol),stat=ierr)
 if (ierr /= 0 ) then
   write (message,'(3a)')' integrate_gamma_tr : ERROR- ',ch10,&
&   ' trying to allocate array tmp_gkkout '
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if
 allocate (tmp_etain (elph_ds%ngkkband*elph_ds%ngkkband,elph_ds%nFSkpt,elph_ds%nsppol),stat=ierr)
 if (ierr /= 0 ) then
   write (message,'(3a)')' integrate_gamma_tr : ERROR- ',ch10,&
&   ' trying to allocate array tmp_etain '
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if
 allocate (tmp_etaout (elph_ds%ngkkband*elph_ds%ngkkband,elph_ds%nFSkpt,elph_ds%nsppol),stat=ierr)
 if (ierr /= 0 ) then
   write (message,'(3a)')' integrate_gamma_tr : ERROR- ',ch10,&
&   ' trying to allocate array tmp_etaout '
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if

 do iqpt=1,elph_ds%nqptirred
   write(6,*)'iqpt',iqpt

   if (elph_ds%gkqwrite == 0) then
     tmp_gkk = elph_ds%gkk_qpt(:,:,:,:,:,iqpt)
     tmp_etain = elph_tr_ds%eta_trin(:,:,:,iqpt)
     tmp_etaout= elph_tr_ds%eta_trout(:,:,:,iqpt)

   else if (elph_ds%gkqwrite == 1) then
     do iFSkpt=1, elph_ds%nFSkpt
       read(elph_ds%unitgkq,REC=((iqpt-1)*elph_ds%nFSkpt+iFSkpt)) tmp_gkk(:,:,:,iFSkpt,:)
     end do
     do iFSkpt=1,elph_ds%nFSkpt
       read (elph_tr_ds%unitgkq_trin,REC=((iqpt-1)*elph_ds%nFSkpt+iFSkpt)) tmp_etain(:,iFSkpt,:)
     end do
     do iFSkpt=1,elph_ds%nFSkpt
       read (elph_tr_ds%unitgkq_trout,REC=((iqpt-1)*elph_ds%nFSkpt+iFSkpt)) tmp_etaout(:,iFSkpt,:)
     end do
     write(6,*)'read iqpt OK'
   end if

   iqpt_fullbz = elph_ds%qirredtofull(iqpt)
   
   do isppol=1,elph_ds%nsppol  
     do iFSkpt=1,elph_ds%nFSkpt
       iFSkptq = FSfullpqtofull(iFSkpt,iqpt_fullbz)
       
       do ib1=1,elph_ds%ngkkband
         do ib2=1,elph_ds%ngkkband
           ibeff=ib2+(ib1-1)*elph_ds%ngkkband
           elph_tr_ds%gamma_qpt_trin(:,:,isppol,iqpt_fullbz) = elph_tr_ds%gamma_qpt_trin(:,:,isppol,iqpt_fullbz) + &
&           tmp_gkk(:,ibeff,:,iFSkpt,isppol)&
&           *tmp_etain(ibeff,iFSkpt,isppol)&
&           *elph_ds%gkk_intweight(ib1,iFSkpt,isppol)*elph_ds%gkk_intweight(ib2,iFSkptq,isppol)
           
           elph_tr_ds%gamma_qpt_trout(:,:,isppol,iqpt_fullbz) = elph_tr_ds%gamma_qpt_trout(:,:,isppol,iqpt_fullbz) + &
&           tmp_gkk(:,ibeff,:,iFSkpt,isppol)&
&           *tmp_etaout(ibeff,iFSkpt,isppol)&
&           *elph_ds%gkk_intweight(ib1,iFSkpt,isppol)*elph_ds%gkk_intweight(ib2,iFSkptq,isppol)
         end do
       end do
     end do ! ik
   end do ! isppol
 end do ! iq

 deallocate (tmp_gkk)
 deallocate (tmp_etain)
 deallocate (tmp_etaout)


!need prefactor of 1/nkpt for each integration over 1 kpoint index.
!NOT INCLUDED IN elph_ds%gkk_intweight
 do iqpt=1,elph_ds%nqpt_full
   elph_tr_ds%gamma_qpt_trout(:,:,:,iqpt) = elph_tr_ds%gamma_qpt_trout(:,:,:,iqpt) * elph_ds%occ_factor / elph_ds%nFSkpt
   elph_tr_ds%gamma_qpt_trin(:,:,:,iqpt) = elph_tr_ds%gamma_qpt_trin(:,:,:,iqpt) * elph_ds%occ_factor / elph_ds%nFSkpt
 end do

 write (message,'(2a)')' integrate_gamma_tr : transport gamma matrices are calculated ',&
& ' in recip space and for irred qpoints'
 call wrtout(std_out,message,'COLL')

end subroutine integrate_gamma_tr
!!***
